//
//  GLStyle.h
//  GLMapKit
//
//  Created by Jeffrey Early on 12/30/06.
//  Copyright 2006 __MyCompanyName__. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import <GLGeometryKit/GLDrawContext.h>
#import <GLGeometryKit/GLSpatialObject.h>

@class GLGeometry, GLRule;

@interface GLStyle : NSObject <NSCoding, NSCopying> {
	NSString *name;
	NSString *title;
	NSString *abstract;
	NSMutableArray *rules;
}

@property(copy) NSString *name, *title, *abstract;
@property(readonly, strong) NSMutableArray *rules;

// returns a the default style object for a given geometry type.
- (id) initForGeometry: (GLGeometry *) geometry;
+ (GLStyle *) styleForGeometry: (GLGeometry *) geometry;

// Tests for complete equality, all properties, all rules, and all symbolizers are equal
- (BOOL) isEqualToStyle: (GLStyle *) aStyle;

// Only requires that the two styles share the same rules, with the same type of symbolizers in the same order
- (BOOL) isStructurallyEqualToStyle: (GLStyle *) aStyle;

- (BOOL) addRule: (GLRule *) rule;
- (BOOL) removeRule: (GLRule *) rule;
- (BOOL) addRule: (GLRule *) rule atIndex: (NSUInteger) index;
- (BOOL) moveRuleFromIndex: (NSUInteger) oldIndex toIndex: (NSUInteger) newIndex;


/************************************************/
/*		Controller								*/
/************************************************/

#pragma mark -
#pragma mark Controller
#pragma mark

// The simplest styles act only as a views for spatial object's geometry. However, the functionality
// can be extended to include interaction between the view and the spatial object. Such interaction
// is turned off by default, but can be overridden with the methods below.

// If YES is returned, the users mouse over will indicate the object can be selected and the MapView
// will attemt to change the current selection with the object's controller.
// If NO is returned, the MapView will not even *attempt* to change the selection with the object's controller.
//- (BOOL) isSelectable;

// Specify zero or more items to recieve different events.
- (GLSpatialObjectEventOptions) spatialObjectEventOptions;

// Depending on the request, these functions may be called.
- (void) mouseDown: (NSEvent *) event atPoint: (GLPoint) aPoint forSpatialObject: (NSObject <GLSpatialObject> *) spatialObject inContext: (NSObject <GLDrawContext> *) drawContext;
- (void) mouseUp: (NSEvent *) event atPoint: (GLPoint) aPoint forSpatialObject: (id <GLSpatialObject>) spatialObject inContext: (NSObject <GLDrawContext> *) drawContext;
- (void) mouseDragged: (NSEvent *) event atPoint: (GLPoint) aPoint forSpatialObject: (id <GLSpatialObject>) spatialObject inContext: (NSObject <GLDrawContext> *) drawContext;
- (void) mouseEntered: (NSEvent *) event atPoint: (GLPoint) aPoint forSpatialObject: (id <GLSpatialObject>) spatialObject inContext: (NSObject <GLDrawContext> *) drawContext;
- (void) mouseExited: (NSEvent *) event atPoint: (GLPoint) aPoint forSpatialObject: (id <GLSpatialObject>) spatialObject inContext: (NSObject <GLDrawContext> *) drawContext;
- (void) mouseMoved: (NSEvent *) event atPoint: (GLPoint) aPoint forSpatialObject: (id <GLSpatialObject>) spatialObject inContext: (NSObject <GLDrawContext> *) drawContext;

/************************************************/
/*		Spatial Object Draw Additions			*/
/************************************************/

#pragma mark -
#pragma mark Spatial Object Draw Additions
#pragma mark

- (void) didRemoveFromContext: (NSObject <GLDrawContext> *) drawContext forSpatialObject: (id <GLSpatialObject>) spatialObject;

// Sorts through its rules and symbolizes the spatial object in the context.
// See the Styled Layer Descriptor Implementation Specification for information on how the rules are applied.
- (void) drawSpatialObject: (id <GLSpatialObject>) spatialObject inContext: (NSObject <GLDrawContext> *) drawContext selected: (BOOL) isSelected;
 
- (BOOL) isSpatialObject: (id <GLSpatialObject>) spatialObject atPoint: (GLPoint) aPoint inContext: (NSObject <GLDrawContext> *) drawContext selected: (BOOL) isSelected;

@end
